{ (* HEADER *) 

open Parser;;

exception UnknownChar;;

}

(* REGULAR DEFINITIONS *)

let integer = '-'? ['0'-'9']+
let real    = '-'? ['0'-'9']+['.']['0'-'9']*
let ident   = ['a'-'z' 'A'-'Z' '_'] ['a'-'z' 'A'-'Z' '0'-'9' '_']*

(* RULES *)

rule lex =
  parse integer         { NAT (int_of_string (Lexing.lexeme lexbuf)) }
      | real            { REAL(float_of_string (Lexing.lexeme lexbuf)) }

      | "true"          { TRUE }
      | "false"         { FALSE }

      | "program"       { PROGRAM }
      | "var"           { VAR }
      | "const"         { CONST }
      | "int"           { INT }
      | "float"         { FLOAT }
      | "begin"         { BEGIN }
      | "end"           { END }
      | "if"            { IF }
      | "then"          { THEN }
      | "else"          { ELSE }
      | "while"         { WHILE }
      | "do"            { DO }
      | "for"           { FOR }
      | "to"            { TO }
      | "write"         { WRITE }

      | ident           { IDE (Lexing.lexeme lexbuf) }

      | "+"             { PLUS }
      | "-"             { MINUS }
      | "*"             { TIMES }
      | "/"             { DIVISION }
      | "="             { EQUAL }
      | "<="            { LESSEQUAL }
      | "<"             { LESS }
      | "&"             { AND }
      | "|"             { OR }
      | "!"             { NOT }
      | ":="            { ASSIGN }


      | ";"             { SEMICOLON }   
      | ":"             { COLON }   

      | "("             { LP }
      | ")"             { RP }

      | [' ' '\t' '\n'] { lex lexbuf }
      | eof             { EOF }

      | _               { raise UnknownChar }
